@@ -203,6 +203,7 @@ usage()
     main_msg("-font <font>\t\tUse <font> for normal text (also: -fn)");
     main_msg("-geometry <geom>\tUse <geom> for initial geometry (also: -geom)");
     main_msg("-reverse\t\tUse reverse video (also: -rv)");
+    main_msg("-display <display>\tRun vim on <display> (also: --display)");
 #endif
     mch_windexit(1);
 }
@@ -309,6 +310,10 @@ main
     setlocale(LC_ALL, "");	/* for ctype() and the like */
 #endif
 
+#ifdef MSWIN
+    win32_init();		/* init toupper() and tolower() tables */
+#endif
+
 #ifdef USE_GUI_MAC
     /* Macinthosh needs this before any memory is allocated. */
     gui_prepare(&argc, argv);	/* Prepare for possibly starting GUI sometime */
@@ -329,7 +334,11 @@ main
      */
     for (i = 0; i < argc; i++)
     {
-	if (STRCMP(argv[i], "-display") == 0)
+	if (STRCMP(argv[i], "-display") == 0
+#ifdef USE_GUI_GTK
+		|| STRCMP(argv[i], "--display") == 0
+#endif
+		)
 	{
 	    xterm_display = argv[i + 1];
 	    break;
@@ -1365,12 +1374,12 @@ main
 #ifdef AUTOCMD
     --autocmd_no_enter;
 #endif
-    win_enter(firstwin, FALSE);		    /* back to first window */
+    win_enter(firstwin, FALSE);		/* back to first window */
 #ifdef AUTOCMD
     --autocmd_no_leave;
 #endif
     if (window_count > 1)
-	win_equal(curwin, FALSE);	    /* adjust heights */
+	win_equal(curwin, FALSE);	/* adjust heights */
 
     /*
      * If there are more file names in the argument list than windows,
